@@ -1,6 +1,6 @@
 # Vend::Table::Common - Common access methods for Interchange databases
 #
-# $Id: Common.pm,v 2.50 2008-05-06 20:42:59 markj Exp $
+# $Id: Common.pm,v 2.51 2008-05-26 02:30:04 markj Exp $
 #
 # Copyright (C) 2002-2008 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
@@ -23,7 +23,7 @@
 # Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 # MA  02110-1301  USA.
 
-$VERSION = substr(q$Revision: 2.50 $, 10);
+$VERSION = substr(q$Revision: 2.51 $, 10);
 use strict;
 
 package Vend::Table::Common;
@@ -416,6 +416,9 @@ sub set_slice {
 
 	if(ref $fary ne 'ARRAY') {
 		my $href = $fary;
+		if(ref $href ne 'HASH') {
+			$href = { splice (@_, 2) };
+		}
 		$vary = [ values %$href ];
 		$fary = [ keys   %$href ];
 	}
@@ -1301,7 +1304,7 @@ my %format = (
         while (<IN>) {
             chomp;
 			\@fields = ();
-			s/\\r?\\n\\r?\\n([\\000-\\377]*)//
+			s/\\r?\\n\\r?\\n((?s:.)*)//
 				and \$fields[$idx] = \$1;
 
 			while(s!($codere):[ \\t]*(.*)\\n?!!) {
